// tslint:disable:prefer-for-of
import { ArrayPixelBlob } from "./array-pixel-blob";
import { quantize, QuantizedColor } from "./color-quantization";
import { test200ImageData } from "../__test-images__/test200";
import { testGrey200ImageData } from "../__test-images__/testGrey200";

describe("Generating a histogram from an image", (): void => {
    test("quantize with default settings", () => {
        const pixels: ArrayPixelBlob = new ArrayPixelBlob(test200ImageData, 200, 133);

        const results: QuantizedColor[] = quantize(pixels);

        expect(results.length).toBe(64);
        expect(results[0].color.toStringHexRGB()).toBe("#8f98a7");
        expect(results[0].pixelCount).toBe(93);
        expect(results[0].colorVolume).toBe(245);
        expect(results[1].color.toStringHexRGB()).toBe("#094b58");
        expect(results[1].pixelCount).toBe(271);
        expect(results[1].colorVolume).toBe(80);
        expect(results[2].color.toStringHexRGB()).toBe("#e7b8b3");
        expect(results[2].pixelCount).toBe(142);
        expect(results[2].colorVolume).toBe(144);
        expect(results[3].color.toStringHexRGB()).toBe("#bfc2cd");
        expect(results[3].pixelCount).toBe(252);
        expect(results[3].colorVolume).toBe(80);
        expect(results[4].color.toStringHexRGB()).toBe("#b87264");
        expect(results[4].pixelCount).toBe(131);
        expect(results[4].colorVolume).toBe(150);
        expect(results[5].color.toStringHexRGB()).toBe("#594747");
        expect(results[5].pixelCount).toBe(240);
        expect(results[5].colorVolume).toBe(80);
        expect(results[6].color.toStringHexRGB()).toBe("#5f6f7a");
        expect(results[6].pixelCount).toBe(113);
        expect(results[6].colorVolume).toBe(144);
        expect(results[7].color.toStringHexRGB()).toBe("#7b5453");
        expect(results[7].pixelCount).toBe(268);
        expect(results[7].colorVolume).toBe(60);
        expect(results[8].color.toStringHexRGB()).toBe("#b58b89");
        expect(results[8].pixelCount).toBe(124);
        expect(results[8].colorVolume).toBe(126);
        expect(results[9].color.toStringHexRGB()).toBe("#ac5c4d");
        expect(results[9].pixelCount).toBe(6);
        expect(results[9].colorVolume).toBe(2299);
        expect(results[10].color.toStringHexRGB()).toBe("#8e7d84");
        expect(results[10].pixelCount).toBe(162);
        expect(results[10].colorVolume).toBe(84);
        expect(results[11].color.toStringHexRGB()).toBe("#538ba1");
        expect(results[11].pixelCount).toBe(44);
        expect(results[11].colorVolume).toBe(297);
        expect(results[12].color.toStringHexRGB()).toBe("#3c322f");
        expect(results[12].pixelCount).toBe(276);
        expect(results[12].colorVolume).toBe(45);
        expect(results[13].color.toStringHexRGB()).toBe("#2e4443");
        expect(results[13].pixelCount).toBe(37);
        expect(results[13].colorVolume).toBe(324);
        expect(results[14].color.toStringHexRGB()).toBe("#105e6e");
        expect(results[14].pixelCount).toBe(188);
        expect(results[14].colorVolume).toBe(60);
        expect(results[15].color.toStringHexRGB()).toBe("#5f6463");
        expect(results[15].pixelCount).toBe(12);
        expect(results[15].colorVolume).toBe(910);
        expect(results[16].color.toStringHexRGB()).toBe("#9d7670");
        expect(results[16].pixelCount).toBe(64);
        expect(results[16].colorVolume).toBe(168);
        expect(results[17].color.toStringHexRGB()).toBe("#de9d97");
        expect(results[17].pixelCount).toBe(179);
        expect(results[17].colorVolume).toBe(60);
        expect(results[18].color.toStringHexRGB()).toBe("#083c4c");
        expect(results[18].pixelCount).toBe(41);
        expect(results[18].colorVolume).toBe(252);
        expect(results[19].color.toStringHexRGB()).toBe("#446875");
        expect(results[19].pixelCount).toBe(90);
        expect(results[19].colorVolume).toBe(108);
        expect(results[20].color.toStringHexRGB()).toBe("#3b4f57");
        expect(results[20].pixelCount).toBe(136);
        expect(results[20].colorVolume).toBe(64);
        expect(results[21].color.toStringHexRGB()).toBe("#c89089");
        expect(results[21].pixelCount).toBe(72);
        expect(results[21].colorVolume).toBe(112);
        expect(results[22].color.toStringHexRGB()).toBe("#06343c");
        expect(results[22].pixelCount).toBe(297);
        expect(results[22].colorVolume).toBe(27);
        expect(results[23].color.toStringHexRGB()).toBe("#202727");
        expect(results[23].pixelCount).toBe(151);
        expect(results[23].colorVolume).toBe(50);
        expect(results[24].color.toStringHexRGB()).toBe("#e36a27");
        expect(results[24].pixelCount).toBe(19);
        expect(results[24].colorVolume).toBe(396);
        expect(results[25].color.toStringHexRGB()).toBe("#9f8c8c");
        expect(results[25].pixelCount).toBe(5);
        expect(results[25].colorVolume).toBe(1440);
        expect(results[26].color.toStringHexRGB()).toBe("#804734");
        expect(results[26].pixelCount).toBe(9);
        expect(results[26].colorVolume).toBe(784);
        expect(results[27].color.toStringHexRGB()).toBe("#faad94");
        expect(results[27].pixelCount).toBe(113);
        expect(results[27].colorVolume).toBe(60);
        expect(results[28].color.toStringHexRGB()).toBe("#a0635b");
        expect(results[28].pixelCount).toBe(89);
        expect(results[28].colorVolume).toBe(72);
        expect(results[29].color.toStringHexRGB()).toBe("#b0543b");
        expect(results[29].pixelCount).toBe(6);
        expect(results[29].colorVolume).toBe(990);
        expect(results[30].color.toStringHexRGB()).toBe("#503a32");
        expect(results[30].pixelCount).toBe(92);
        expect(results[30].colorVolume).toBe(60);
        expect(results[31].color.toStringHexRGB()).toBe("#306371");
        expect(results[31].pixelCount).toBe(71);
        expect(results[31].colorVolume).toBe(72);
        expect(results[32].color.toStringHexRGB()).toBe("#8f6a6d");
        expect(results[32].pixelCount).toBe(59);
        expect(results[32].colorVolume).toBe(84);
        expect(results[33].color.toStringHexRGB()).toBe("#595960");
        expect(results[33].pixelCount).toBe(92);
        expect(results[33].colorVolume).toBe(50);
        expect(results[34].color.toStringHexRGB()).toBe("#8a655f");
        expect(results[34].pixelCount).toBe(74);
        expect(results[34].colorVolume).toBe(60);
        expect(results[35].color.toStringHexRGB()).toBe("#352424");
        expect(results[35].pixelCount).toBe(69);
        expect(results[35].colorVolume).toBe(60);
        expect(results[36].color.toStringHexRGB()).toBe("#b57a76");
        expect(results[36].pixelCount).toBe(40);
        expect(results[36].colorVolume).toBe(88);
        expect(results[37].color.toStringHexRGB()).toBe("#b7b4c4");
        expect(results[37].pixelCount).toBe(17);
        expect(results[37].colorVolume).toBe(196);
        expect(results[38].color.toStringHexRGB()).toBe("#ac7c84");
        expect(results[38].pixelCount).toBe(6);
        expect(results[38].colorVolume).toBe(528);
        expect(results[39].color.toStringHexRGB()).toBe("#fba078");
        expect(results[39].pixelCount).toBe(41);
        expect(results[39].colorVolume).toBe(72);
        expect(results[40].color.toStringHexRGB()).toBe("#8f5753");
        expect(results[40].pixelCount).toBe(63);
        expect(results[40].colorVolume).toBe(40);
        expect(results[41].color.toStringHexRGB()).toBe("#1b1c15");
        expect(results[41].pixelCount).toBe(205);
        expect(results[41].colorVolume).toBe(12);
        expect(results[42].color.toStringHexRGB()).toBe("#2c201b");
        expect(results[42].pixelCount).toBe(111);
        expect(results[42].colorVolume).toBe(20);
        expect(results[43].color.toStringHexRGB()).toBe("#d88b7f");
        expect(results[43].pixelCount).toBe(25);
        expect(results[43].colorVolume).toBe(84);
        expect(results[44].color.toStringHexRGB()).toBe("#688ca0");
        expect(results[44].pixelCount).toBe(16);
        expect(results[44].colorVolume).toBe(126);
        expect(results[45].color.toStringHexRGB()).toBe("#a3b4c4");
        expect(results[45].pixelCount).toBe(9);
        expect(results[45].colorVolume).toBe(196);
        expect(results[46].color.toStringHexRGB()).toBe("#16160c");
        expect(results[46].pixelCount).toBe(92);
        expect(results[46].colorVolume).toBe(18);
        expect(results[47].color.toStringHexRGB()).toBe("#6c4d4b");
        expect(results[47].pixelCount).toBe(79);
        expect(results[47].colorVolume).toBe(20);
        expect(results[48].color.toStringHexRGB()).toBe("#d6bfc5");
        expect(results[48].pixelCount).toBe(49);
        expect(results[48].colorVolume).toBe(32);
        expect(results[49].color.toStringHexRGB()).toBe("#24363a");
        expect(results[49].pixelCount).toBe(58);
        expect(results[49].colorVolume).toBe(27);
        expect(results[50].color.toStringHexRGB()).toBe("#e5c4bc");
        expect(results[50].pixelCount).toBe(6);
        expect(results[50].colorVolume).toBe(240);
        expect(results[51].color.toStringHexRGB()).toBe("#35241c");
        expect(results[51].pixelCount).toBe(17);
        expect(results[51].colorVolume).toBe(84);
        expect(results[52].color.toStringHexRGB()).toBe("#ed8a65");
        expect(results[52].pixelCount).toBe(7);
        expect(results[52].colorVolume).toBe(200);
        expect(results[53].color.toStringHexRGB()).toBe("#8c8c94");
        expect(results[53].pixelCount).toBe(16);
        expect(results[53].colorVolume).toBe(80);
        expect(results[54].color.toStringHexRGB()).toBe("#04232a");
        expect(results[54].pixelCount).toBe(41);
        expect(results[54].colorVolume).toBe(30);
        expect(results[55].color.toStringHexRGB()).toBe("#e3aca3");
        expect(results[55].pixelCount).toBe(18);
        expect(results[55].colorVolume).toBe(60);
        expect(results[56].color.toStringHexRGB()).toBe("#0c1411");
        expect(results[56].pixelCount).toBe(59);
        expect(results[56].colorVolume).toBe(16);
        expect(results[57].color.toStringHexRGB()).toBe("#241d18");
        expect(results[57].pixelCount).toBe(154);
        expect(results[57].colorVolume).toBe(6);
        expect(results[58].color.toStringHexRGB()).toBe("#042434");
        expect(results[58].pixelCount).toBe(10);
        expect(results[58].colorVolume).toBe(72);
        expect(results[59].color.toStringHexRGB()).toBe("#263429");
        expect(results[59].pixelCount).toBe(8);
        expect(results[59].colorVolume).toBe(72);
        expect(results[60].color.toStringHexRGB()).toBe("#b3a5ac");
        expect(results[60].pixelCount).toBe(8);
        expect(results[60].colorVolume).toBe(54);
        expect(results[61].color.toStringHexRGB()).toBe("#82473c");
        expect(results[61].pixelCount).toBe(17);
        expect(results[61].colorVolume).toBe(20);
        expect(results[62].color.toStringHexRGB()).toBe("#242c1a");
        expect(results[62].pixelCount).toBe(12);
        expect(results[62].colorVolume).toBe(16);
        expect(results[63].color.toStringHexRGB()).toBe("#150c0c");
        expect(results[63].pixelCount).toBe(15);
        expect(results[63].colorVolume).toBe(12);
    });
    test("quantize a greyscale image", () => {
        const pixels: ArrayPixelBlob = new ArrayPixelBlob(testGrey200ImageData, 200, 125);

        const results: QuantizedColor[] = quantize(pixels);

        expect(results.length).toBe(24);
        expect(results[0].color.toStringHexRGB()).toBe("#1c1c1c");
        expect(results[0].pixelCount).toBe(475);
        expect(results[0].colorVolume).toBe(1);
        expect(results[1].color.toStringHexRGB()).toBe("#141414");
        expect(results[1].pixelCount).toBe(452);
        expect(results[1].colorVolume).toBe(1);
        expect(results[2].color.toStringHexRGB()).toBe("#242424");
        expect(results[2].pixelCount).toBe(439);
        expect(results[2].colorVolume).toBe(1);
        expect(results[3].color.toStringHexRGB()).toBe("#6c6c6c");
        expect(results[3].pixelCount).toBe(135);
        expect(results[3].colorVolume).toBe(3);
        expect(results[4].color.toStringHexRGB()).toBe("#2c2c2c");
        expect(results[4].pixelCount).toBe(398);
        expect(results[4].colorVolume).toBe(1);
        expect(results[5].color.toStringHexRGB()).toBe("#bdbdbd");
        expect(results[5].pixelCount).toBe(9);
        expect(results[5].colorVolume).toBe(42);
        expect(results[6].color.toStringHexRGB()).toBe("#343434");
        expect(results[6].pixelCount).toBe(344);
        expect(results[6].colorVolume).toBe(1);
        expect(results[7].color.toStringHexRGB()).toBe("#3c3c3c");
        expect(results[7].pixelCount).toBe(340);
        expect(results[7].colorVolume).toBe(1);
        expect(results[8].color.toStringHexRGB()).toBe("#acacac");
        expect(results[8].pixelCount).toBe(21);
        expect(results[8].colorVolume).toBe(16);
        expect(results[9].color.toStringHexRGB()).toBe("#646464");
        expect(results[9].pixelCount).toBe(167);
        expect(results[9].colorVolume).toBe(2);
        expect(results[10].color.toStringHexRGB()).toBe("#444444");
        expect(results[10].pixelCount).toBe(321);
        expect(results[10].colorVolume).toBe(1);
        expect(results[11].color.toStringHexRGB()).toBe("#0c0c0c");
        expect(results[11].pixelCount).toBe(313);
        expect(results[11].colorVolume).toBe(1);
        expect(results[12].color.toStringHexRGB()).toBe("#4c4c4c");
        expect(results[12].pixelCount).toBe(298);
        expect(results[12].colorVolume).toBe(1);
        expect(results[13].color.toStringHexRGB()).toBe("#545454");
        expect(results[13].pixelCount).toBe(292);
        expect(results[13].colorVolume).toBe(1);
        expect(results[14].color.toStringHexRGB()).toBe("#9c9c9c");
        expect(results[14].pixelCount).toBe(48);
        expect(results[14].colorVolume).toBe(6);
        expect(results[15].color.toStringHexRGB()).toBe("#040404");
        expect(results[15].pixelCount).toBe(248);
        expect(results[15].colorVolume).toBe(1);
        expect(results[16].color.toStringHexRGB()).toBe("#5c5c5c");
        expect(results[16].pixelCount).toBe(246);
        expect(results[16].colorVolume).toBe(1);
        expect(results[17].color.toStringHexRGB()).toBe("#949494");
        expect(results[17].pixelCount).toBe(54);
        expect(results[17].colorVolume).toBe(4);
        expect(results[18].color.toStringHexRGB()).toBe("#b4b4b4");
        expect(results[18].pixelCount).toBe(9);
        expect(results[18].colorVolume).toBe(15);
        expect(results[19].color.toStringHexRGB()).toBe("#a4a4a4");
        expect(results[19].pixelCount).toBe(32);
        expect(results[19].colorVolume).toBe(4);
        expect(results[20].color.toStringHexRGB()).toBe("#747474");
        expect(results[20].pixelCount).toBe(121);
        expect(results[20].colorVolume).toBe(1);
        expect(results[21].color.toStringHexRGB()).toBe("#8c8c8c");
        expect(results[21].pixelCount).toBe(54);
        expect(results[21].colorVolume).toBe(2);
        expect(results[22].color.toStringHexRGB()).toBe("#7c7c7c");
        expect(results[22].pixelCount).toBe(102);
        expect(results[22].colorVolume).toBe(1);
        expect(results[23].color.toStringHexRGB()).toBe("#848484");
        expect(results[23].pixelCount).toBe(82);
        expect(results[23].colorVolume).toBe(1);
    });
});
